# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <willmcgugan+2d@gmail.com>.
# All Rights Reserved.
#
# This file is part of sorethumb,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

import os.path
from PIL import Image
import hashlib

from sorethumb.filters import Filter

class ThumbError(Exception):
    pass


def _camel_to_lower(name):
    """ Converts CamelCase to lower_case_with_underscores """
    return ''.join([c if not c.isupper() else '_'+c.lower()
                    for c in name]).lstrip('_')

class ThumbnailBase(type):

    """ This is a metaclass which extracts a thumbnail declaration and saves
    it in to a global registry. Not to be used directly, see Thumbnail below.

    """

    thumb_processors = {}

    def __new__(cls, name, bases, attrs):

        parents = [b for b in bases if isinstance(b, ThumbnailBase)]
        if not parents:
            return type.__new__(cls, name, bases, attrs)

        if 'name' not in attrs:
            attrs['name'] = _camel_to_lower(name)

        def check_get(name):
            """ Retrieves a value from attrs or throws an error if it does
            not exist. """
            if name not in attrs:
                raise ValueError("'%s' is a required field for Thumbnails" % name)
            return attrs[name]

        thumb_name = check_get('name')
        thumb_version = str(attrs.get('version', '1'))

        # Find all the filters and store them in a list
        # Also store the extra fields in processor_data
        processor_data = {}
        for k, v in attrs.items():
            if not k.startswith('__'):
                processor_data[k] = v

        settings = {}
        if 'Settings' in attrs:
            settings = dict([(k, v)
                             for k, v in attrs['Settings'].__dict__.items()
                                if not k.startswith('_')])

        filters = attrs.get('filters', [])

        processor_data.update(name = thumb_name,
                              filters = filters)

        processor_class = type.__new__(cls, name, bases, attrs)

        thumb_obj = processor_class(thumb_name, thumb_version, processor_data)

        thumb_obj.settings = settings
        ThumbnailBase.thumb_processors[thumb_name] = thumb_obj

        return processor_class



class Thumbnail(metaclass=ThumbnailBase):

    """ This class is used to declare how images are turned in to thumbnails.
    It also exposes the basic API to retrieve processors from the global
    registry. Generally though, the 'sorethumb' django tag should be used.

    """

    def get_setting(self, name, default=None):
        return self.settings.get(name, default)

    def __init__(self, name, version, processor_data):
        self.name = name
        self.version = version
        self.processor_data = processor_data
        self.filters = self.processor_data['filters']
        self._dir_name = self._get_dir_name()

    @classmethod
    def get_processor(self, name):
        if name not in ThumbnailBase.thumb_processors:
            raise ThumbError("'%s' is not a registered thumb processor" % name)
        return ThumbnailBase.thumb_processors[name]

    @classmethod
    def render(self, processor_name, image_path):
        processor = self.get_processor(processor_name)
        return processor.process(image_path)


    def _get_dir_name(self):
        """ Gets a string that identifies the sequence of filters. """

        def get_filter_name(filter):
            if hasattr(filter, 'name'):
                return filter.name
            return filter.__class__.__name__

        filter_params = '.'.join([get_filter_name(filter) + ':' + repr(filter.get_params())
                                  for filter in self.filters])
        filter_hash = hashlib.md5(filter_params.encode('utf-8')).hexdigest()[::2]
        name = "%s.%s.v%s" % (self.name, filter_hash, self.version)
        return name

    def get_missing_image(self):
        """Return a substitute image if the given image path does not exist
        or can not be read. May be overriden for different behaviour."""
        img = Image.new("RGB", (100, 100), (255, 0, 255))
        return img

    def process(self, image_path, format=None):

        """ Produces a thumbnail from a given path by running it through each
        registered filter in turn (if the thumbnail doesn't already exist).
        Returns the url path to the the thumbnail.

        """
        def url_join(u1, u2):
            return '/'.join((u1.rstrip('/'), u2.lstrip('/')))

        image_path = image_path or ''
        image_root = self.get_setting('SORETHUMB_IMAGE_ROOT') or ''
        input_name = image_path[len(image_root):].lstrip('/')

        if not input_name or not image_path:
            input_name = "__default"
            default_image = self.get_setting('SORETHUMB_DEFAULT_IMAGE')
            image_path = self.processor_data.get('default', default_image)

        if not image_path:
            raise ThumbError('No path to original, and SORETHUMB_DEFAULT_IMAGE is not set')

        path = self.get_setting('SORETHUMB_OUTPUT_PATH')
        if not path:
            raise ThumbError('SORETHUMB_OUTPUT_PATH is required')

        # Build a destination path from the filter name
        output_dir = os.path.join(path, self._dir_name)
        if format is None:
            format = self.processor_data.get('format', 'jpg')
        ext = '.thumb.' + format.lstrip('.')
        output_path = url_join(output_dir, input_name + ext)


        url_root = self.get_setting('SORETHUMB_URL_ROOT')
        url_path = os.path.join(url_root, url_join(self._dir_name, input_name + ext))

        # If it exists just return the path to the processed image
        if os.path.exists(output_path):
            return url_path

        # Make the destination folder, if it doesn't exist
        try:
            output_dir = os.path.dirname(output_path)
            os.makedirs(output_dir)
        except OSError:
            pass

        try:
            img = Image.open(image_path)
        except (OSError, IOError) as e:
            # If the image doesn't exist, replace it with a pink square
            # This should only occur when the media isn't up to date
            img = self.get_missing_image()
            output_path = url_join(output_dir, '__missing.png')
            url_path = os.path.join(url_root, url_join(self._dir_name, '__missing.png'))

        # Run the image through all registered filters
        for filter in self.filters:
            img = filter(img) or img

        # Write the processed image and return the path
        try:
            img.save(output_path,
                     quality = self.processor_data.get('quality', 99))

        except (OSError, IOError) as e:
            # Ultra cautious approach to avoid a 500 error
            # If save fails it will be re-attempted next request
            pass

        return url_path





# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <willmcgugan+2d@gmail.com>.
# All Rights Reserved.
#
# This file is part of sorethumb,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""Filters which require the aggdraw module to work

"""

from PIL import Image, ImageDraw
import aggdraw

from sorethumb.filters import Filter, parse_color, html_color


class RoundedCornerFilter(Filter):

    """ Rounds the corners of an image with a given radius, and optionally
    draws a border.

    """

    def __init__(self, radius, border=None):
        """
        :param radius: The radius for the corners
        :param border: The border color as HTML string or tuple
        
        """
        
        Filter.__init__(self, radius, border)
        self.radius = radius
        self.border = parse_color(border)

    def __call__(self, img):

        w, h = img.size
        r = self.radius
        d = r * 2
                        
        circle = Image.new('L', (d, d))
        draw = aggdraw.Draw(circle)

        pen = aggdraw.Pen(255)
        brush = aggdraw.Brush(255)
        draw.ellipse((1, 1, d - 1, d - 1), pen, brush)
        draw.flush()

        # Cut it in to four pieces
        tl = circle.crop((0, 0, r, r)) # Top left
        tr = circle.crop((r, 0, d, r)) # Top right
        bl = circle.crop((0, r, r, d)) # Bottom left
        br = circle.crop((r, r, d, d)) # Bottom right

        # Create an alpha image with the four pieces at the corners
        alpha = Image.new('L', img.size, 255)
        alpha.paste(tl, (0, 0))
        alpha.paste(tr, (w - r, 0))
        alpha.paste(bl, (0, h - r))
        alpha.paste(br, (w - r, h - r))

        # Replace the alpha with our new image
        img = img.convert('RGBA')
        img.putalpha(alpha)

        if self.border is None:
            return img        

        # Draw a border with curved corners
        draw = aggdraw.Draw(img)
        pen = aggdraw.Pen(self.border)

        # The +.5s are so that agg doesn't draw over pixel boundries
        draw.arc((0.5, 0.5, d - .5, d - .5), 90, 180, pen)
        draw.line((r, .5, w - r, .5), pen)

        draw.arc((w - d + .5, 0.5, w - .5, d - .5), 0, 90, pen)
        draw.line((w - .5, r, w - .5, h-r), pen)

        draw.arc((w - d + .5, h - d + .5, w - .5, h - .5), 270, 360, pen)
        draw.line((w - r, h - .5, r, h - .5), pen)

        draw.arc((0.5, h - d + .5, d - .5, h - .5), 180, 270, pen)
        draw.line((0.5, h - r, 0.5, r), pen)

        draw.flush()

        return img




# Django settings for djangoexample project.

import os
def project_path(d):
    path = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(path, d)

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

DATABASE_ENGINE = 'sqlite3'           # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
DATABASE_NAME = 'testdb.db'             # Or path to database file if using sqlite3.
DATABASE_USER = ''             # Not used with sqlite3.
DATABASE_PASSWORD = ''         # Not used with sqlite3.
DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.db',
        'OPTIONS': {

        },
    }
}


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = project_path('media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin-media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = 't12u5mi)m2le(k(e5&5^(6$*5(!##rq8rhve@$-8mt#3mqx#xc'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            # ... some options here ...
        },
    },
]

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    #'django.contrib.sessions.middleware.SessionMiddleware',
    #'django.contrib.auth.middleware.AuthenticationMiddleware',
)

ROOT_URLCONF = 'djangoexample.urls'

TEMPLATE_DIRS = [
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    project_path('templates')
]

INSTALLED_APPS = (
    #'django.contrib.auth',
    #'django.contrib.contenttypes',
    #'django.contrib.sessions',
    #'django.contrib.sites',
    'thumbs',
    'sorethumb',
)

SORETHUMB_OUTPUT_PATH = project_path('media/sorethumboutput')
SORETHUMB_URL_ROOT = '/media/sorethumboutput'
SORETHUMB_IMAGE_ROOT = MEDIA_ROOT + '/uploads'
SORETHUMB_DEFAULT_IMAGE = MEDIA_ROOT + '/user.png'





from django.conf import settings
from django.conf.urls import url
from django.views.static import serve
# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

from thumbs.views import examples

def project_path(d):
    path = os.path.split(__file__)[0]
    return os.path.join(path, d)


urlpatterns = [
    # Example:
    # (r'^djangoexample/', include('djangoexample.foo.urls')),

    # Uncomment the admin/doc line below and add 'django.contrib.admindocs'
    # to INSTALLED_APPS to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    #(r'^admin/', include(admin.site.urls)),

    url(r'^$', examples),

    url(r'^media/(?P<path>.*)', serve,
        {'document_root': settings.MEDIA_ROOT, 'show_indexes':True}),

]



# Generated by Django 2.0.7 on 2018-07-10 03:41

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ThumbTest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image_file', models.FileField(upload_to='uploads')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <willmcgugan+2d@gmail.com>.
# All Rights Reserved.
#
# This file is part of sorethumb,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

__version__ = "1.0.1"



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <willmcgugan+2d@gmail.com>.
# All Rights Reserved.
#
# This file is part of sorethumb,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

from PIL import Image, ImageOps
from math import ceil
from sorethumb.filters import Filter, parse_color, html_color
import hashlib


class ThumbnailFilter(Filter):

    """ Uses PIL's thumbnail method to reduce an image. """

    def __init__(self, width, height):
        """
        :param width: The maximum width of the thumbnail
        :param height: The maximum height of the thumbnail

        """
        Filter.__init__(self, width, height)
        self.new_size = (width, height)

    def __call__(self, img):
        img = img.thumbnail(self.new_size, Image.ANTIALIAS)
        return img


class ResizeFilter(Filter):

    """Resizes an image to new dimensions (may change the aspect ratio"""

    def __init__(self, width, height):
        """
        :param width: The required width of the thumbnail
        :param height: The required height of the thumbnail
        """
        Filter.__init__(self, width, height)
        self.new_size = (width, height)

    def __call__(self, img):
        return img.resize(self.new_size, Image.ANTIALIAS)


class SquareFilter(Filter):

    """Crops an image to be square (from center)"""

    def __call__(self, img):
        w, h = img.size

        if w == h:
            return img

        if w > h:
            x = (w - h) / 2
            return img.crop((x, 0, x + h, h))
        else:
            y = (h - w) / 2
            return img.crop((0, y, w, w + y))


class ResizeCanvasFilter(Filter):

    """Adjusts the canvas size and centers the image."""

    def __init__(self,
                 width,
                 height,
                 background_color='#000',
                 background_opacity=255):
        """
        :param width: Width of resized version
        :param height: Height of resized version
        :param background_color: Color for the canvas background
        :param background_opacity: Opacity for canvas background (255 is opaque)

        """
        Filter.__init__(self, width, height, background_color, background_opacity)
        self.new_size = (width, height)
        self.background_color = parse_color(background_color)
        self.background_opacity = background_opacity


    def __call__(self, img):
        r, g, b = self.background_color
        a = self.background_opacity

        new_img = Image.new('RGBA', self.new_size, (r, g, b, a))
        w, h = img.size
        nw, nh = self.new_size
        x = int((nw - w) / 2)
        y = int((nh - h) / 2)
        new_img.paste(img, (x, y))
        return new_img


class OpaqueFilter(Filter):

    """Renders an image with an alpha channel on to an opaque background"""

    def __init__(self, col):
        """
        :param col: The colour of the background as an HTML string or tuple

        """
        Filter.__init__(self, col)
        self.col = parse_color(col)

    def __call__(self, img):
        opaque = Image.new('RGBA', img.size, self.col)
        opaque.paste(img, (0, 0), img)
        opaque = opaque.convert('RGB')
        return opaque


class VerticalGradientFilter(Filter):

    """Renders a vertical gradient from colour 1 to colour 2"""

    def __init__(self, col1, col2):
        """
        :param col1: The colour at the top of the image
        :param col2: The colour at the bottom of the image

        """
        Filter.__init__(self, col1, col2)
        self.col1 = parse_color(col1)
        self.col2 = parse_color(col2)

    def __call__(self, img):
        r1, g1, b1 = map(float, self.col1)
        r2, g2, b2 = map(float, self.col2)

        gradient = Image.new('RGB', img.size)

        w, h = img.size
        fh = float(h)
        for y in range(h):
            i = (y / fh)
            r = r1 + (r2 - r1) * i
            g = g1 + (g2 - g1) * i
            b = b1 + (b2 - b1) * i
            row = Image.new('RGB', (w, 1), (int(r), int(g), int(b)))
            gradient.paste(row, (0, y))

        gradient.paste(img, (0, 0), img)
        gradient = gradient.convert('RGB')
        return gradient


class RectangularCropFilter(Filter):
    """Crops an image to the specified x/y ratio"""

    def __init__(self, ratio):
        """
        :param ratio: New aspect ratio for the image

        """
        Filter.__init__(self, ratio)
        self.ratio = ratio * 1.0

    def __call__(self, img):
        current_width, current_height = img.size

        current_ratio = float(current_width) / current_height

        # check to see whether a crop is necessary:
        if current_ratio != self.ratio:

            if current_ratio > self.ratio:
                # we've got an image which is more landscape than the
                # desired image:
                new_width = int(ceil(current_height / self.ratio))
                new_height = current_height

                new_left = (current_width - new_width) / 2
                new_right = new_left + new_width

                new_size = (int(new_left), 0, int(new_right), int(new_height))

            else:
                # we've got an image which is more portrait than the
                # desired image:
                new_width = current_width
                new_height = int(ceil(current_width / self.ratio))

                new_top = (current_height - new_height) / 2
                new_bottom = new_top + new_height

                new_size = (0, int(new_top), int(new_width), int(new_bottom))

            img = img.crop(new_size)

        return img


class FixedWidthFilter(Filter):
    """Resizes the image to the specified width respecting the aspect ratio"""

    def __init__(self, width, no_upscale=False):
        """
        :param width: New width
        :param no_upscale: If True the image will only be scaled down

        """
        Filter.__init__(self, width, no_upscale)
        self.width = int(width)
        self.no_upscale = bool(no_upscale)

    def __call__(self, img):
        orig_width, orig_height = img.size

        if self.no_upscale and orig_width <= self.width:
            return img

        orig_ratio = float(orig_width) / orig_height

        new_height = int(self.width / orig_ratio)
        img = img.resize((self.width, new_height), Image.ANTIALIAS)

        return img


class FixedHeightFilter(Filter):
    """ Resizes the image to the specified height respecting the aspect ratio """

    def __init__(self, height, no_upscale=False):
        """
        :param height: New height in pixels
        :param no_upscale: If True the image will only be scaled down

        """
        Filter.__init__(self, height, no_upscale)
        self.height = int(height)
        self.no_upscale = bool(no_upscale)

    def __call__(self, img):
        orig_width, orig_height = img.size

        if self.no_upscale and orig_height <= self.height:
            return img

        orig_ratio = float(orig_width) / orig_height
        new_width = int(self.height * orig_ratio)
        img = img.resize((new_width, self.height), Image.ANTIALIAS)

        return img


class OverlayFilter(Filter):
    """ Renders a translucent image over the image """

    def __init__(self, overlay_path):
        """
        :param overlay_path: System path to the overlay image

        """
        Filter.__init__(self, overlay_path)
        self.overlay_path = overlay_path

    def __call__(self, img):
        overlay_img = Image.open(self.overlay_path)
        img.paste(overlay_img, (0, 0), overlay_img)
        return img


class MaskFilter(Filter):
    """ Replaces the alpha channel with a new image """

    def __init__(self, mask_path):
        """
        :param mask_path: Path to mask image

        """
        Filter.__init__(self, mask_path)
        self.mask_path = mask_path

    def __call__(self, img):

        mask = Image.open(self.mask_path)
        mask = mask.convert('L')
        img.convert('RGBA')
        img.putalpha(mask)
        return img

        mask = Image.open(self.mask_path)
        img = img.convert("RGBA")
        r, g, b, a = mask.split()
        img.paste(mask, mask=mask)
        img = img.convert('RGB')
        return img


class OpacityFilter(Filter):
    """Sets the alpha channel to a given opacity"""

    def __init__(self, opacity=.5):
        """
        :param opacity: A floating point value between 0 and 1, where 0

        """
        Filter.__init__(self, opacity)
        self.opacity = opacity

    def __call__(self, img):
        opacity = max(0, min(255, int(self.opacity * 255.0)))
        img.putalpha(opacity)
        return img


class GrayscaleFilter(Filter):
    """Converts the image to grayscale"""

    def __call__(self, img):
        img = ImageOps.grayscale(img)
        return img


class InvertFilter(Filter):
    """Inverts the colours in the image"""

    def __call__(self, img):
        return ImageOps.invert(img)





# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <willmcgugan+2d@gmail.com>.
# All Rights Reserved.
#
# This file is part of sorethumb,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

def parse_color(col):
    """ Transforms an html style color (ie #fe23dd) in to a tuple of three
    values. Any values which are not strings, are passed through unaltered.

    """
    if isinstance(col, (bytes, str)):
        col = col.lstrip('#')
        try:
            if len(col) == 6:
                return tuple([int(c, 16) for c in (col[0:2], col[2:4], col[4:6])])
            elif len(col) == 3:
                return tuple([int(c, 16) * 17 for c in col])
            else:
                raise ValueError("html colours must be 3 or 6 hex characters")
        except ValueError as e:
            raise ValueError("'%s' is not an html colour (%s)" % (col, e.message))
    return col


def html_color(col):
    """ Generates an html colour from a tuple of three values. """
    return ''.join(['%02x' % c for c in col])


class Filter(object):
    """ The base class for thumb filters. """

    def __init__(self, *args, **kwargs):
        self._params = (args, kwargs)

    def get_params(self):
        return self._params

    def __call__(self, img):
        """ The call method should return a PIL image instance (or the same one)
        or None, if no changes are required.

        """
        raise NotImplementedError





from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.conf import settings

from .models import ThumbTest

from sorethumb.filters.defaultfilters import *
from sorethumb.filters.drawfilters import *
from sorethumb.djangothumbnail import DjangoThumbnail


class SmallThumb(DjangoThumbnail):

    filters = [ThumbnailFilter(120, 100)]


class Square(DjangoThumbnail):

    filters = [ThumbnailFilter(100, 100),
              SquareFilter(),]


class RoundedCorners5(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(5)]


class RoundedCorners10(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10)]


class RoundedCorners20(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(20)]


class RoundedCornersEdged(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10, border='#000')]


class RoundedCornersBackground(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10, border='#333'),
               ResizeCanvasFilter(130, 110, '#fff'),
               OpaqueFilter('#fff'),]


class RoundedCornersBackgroundGradient(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10, border='#000'),
               ResizeCanvasFilter(130, 110, '#e2e2ff', background_opacity=0),
               VerticalGradientFilter('#fff', '#88e'),]


# class MaskThumb(DjangoThumbnail):

#     format = 'png'
#     filters = [ThumbnailFilter(120, 100),
#                ResizeCanvasFilter(120, 100, '#000', background_opacity=0),
#                MaskFilter(settings.MEDIA_ROOT+'/alpha.png')]


class GrayThumb(DjangoThumbnail):

    filters = [ThumbnailFilter(120, 100),
               GrayscaleFilter()]


class FadedThumb(DjangoThumbnail):

    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               OpacityFilter(.5)]


# class OverlayThumb(DjangoThumbnail):
#     format = 'png'
#     filters = [ThumbnailFilter(120, 100),
#                OverlayFilter(settings.MEDIA_ROOT+'/user.png')]


thumb_examples = [

    {
    'thumb':'small_thumb',
    'title':'Basic thumbnail',
    'description':"""Here we have a basic thumbnail that uses PIL's thumbnail operation to reduce an image to fit in a defined dimensions.""",

    'code' : '''class SmallThumb(DjangoThumbnail):
    filters = [ThumbnailFilter(120, 100)]'''
    },

    {
    'thumb':'square',
    'title':'Square',
    'description':'As above, but cropped to be square. Since uploaded images can be any old size, they can tend to look ragged when presented in rows. Square thumbs look better in rows, at the expense of a little cropping',

    'code':"""class Square(DjangoThumbnail):
    filters = [ThumbnailFilter(100, 100),
               SquareFilter()] """
    },

    {
    'thumb':'rounded_corners5',
    'title':'5 pixels rounded corner',
    'description':"""Rounded corners without CSS3, on a transparent background. What is it with designers and rounded corners anyway?""",
    'code':"""class RoundedCorners5(DjangoThumbnail):
    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(5)] """,

    },

    {
    'thumb':'rounded_corners10',
    'title':'10 pixels rounded corner',
    'description':'As above, but 10 pixels.',
    },

    {
    'thumb':'rounded_corners20',
    'title':'20 pixels rounded corner',
    'description':'Even more rounded corners',
    },

    {
    'thumb':'rounded_corners_edged',
    'title':'Rounded corners with a border',
    'description':'The rounded corner filter also supports a coloured border',
    'code':"""class RoundedCornersEdged(DjangoThumbnail):
    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10, border='#333')]""",
    },

    {
     'thumb':'rounded_corners_background',
     'title':"Rounded corners on an opaque background",
     'description':"Rounded corners on an opaque backround for browsers with poor support for per-pixel transparency &mdash; IE6 I'm looking at you!",
     'code':"""class RoundedCornersBackground(DjangoThumbnail):
    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10, border='#333'),
               ResizeCanvasFilter(130, 110, '#fff'),
               OpaqueFilter('#fff')] """
     },

    {
     'thumb':'rounded_corners_background_gradient',
     'title':"Rounded corners on a gadient",
     'description':"As above, but on a gradient background. The vertical gradient filter replaces transparent areas with a smooth gradient between two colours.",
     'code':"""class RoundedCornersBackgroundGradient(DjangoThumbnail):
    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               RoundedCornerFilter(10, border='#000'),
               ResizeCanvasFilter(130, 110, '#e2e2ff', background_opacity=0),
               VerticalGradientFilter('#fff', '#88e')]  """
     },

#      {'thumb':'mask_thumb',
#       'title':'Masked thumbnail',
#       'description': 'This thumbnail uses MaskFilter which replaces the alpha channel with another image, to create some interesting effects.',
#       'code':"""class MaskThumb(DjangoThumbnail):
#     format = 'png'
#     filters = [ThumbnailFilter(120, 100),
#                ResizeCanvasFilter(120, 100, '#000', background_opacity=0),
#                MaskFilter(settings.MEDIA_ROOT+'/alpha.png')]
# """
#       },

     {
      'thumb':'gray_thumb',
      'title':'Grayscale',
      'description':'A grayscale thumb, could be used as a hover state.',
      'code':"""class GrayThumb(DjangoThumbnail):
    filters = [ThumbnailFilter(120, 100),
               GrayscaleFilter()]"""
      },

      {
      'thumb':'faded_thumb',
      'title':'50% opacity',
      'description':'The OpacityFilter sets the opacity of the thumbnail.',
      'code':"""class FadedThumb(DjangoThumbnail):
    format = 'png'
    filters = [ThumbnailFilter(120, 100),
               OpacityFilter(.5)] """
      },


    # {
    # 'thumb':'overlay_thumb',
    # 'title':'Thumbnail with overlay',
    # 'description':"""A thumbnail with an overlayed transparent png. Could be used to indicate online status.""",

    # 'code' : '''class OverlayThumb(DjangoThumbnail):
    # format = 'png'
    # filters = [ThumbnailFilter(120, 100),
    #           OverlayFilter(settings.MEDIA_ROOT+'/user.png')]'''
    # },

]

def examples(request):

    context = {'examples':thumb_examples}

    if request.method == 'POST':
        thumb = ThumbTest(image_file=request.FILES.get('file'))
        thumb.save()

    try:
        dbobject = ThumbTest.objects.all().order_by('-pk')[0]
    except IndexError:
        dbobject = None

    context['dbobject'] = dbobject

    return render_to_response('thumbs.html',
                              context,
                              RequestContext(request))



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2degrees Limited <willmcgugan+2d@gmail.com>.
# All Rights Reserved.
#
# This file is part of sorethumb,
# which is subject to the provisions of the BSD at
# <http://dev.2degreesnetwork.com/p/2degrees-license.html>. A copy of the
# license should accompany this distribution. THIS SOFTWARE IS PROVIDED "AS IS"
# AND ANY AND ALL EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT
# NOT LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST
# INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
#
##############################################################################

from urllib.parse import urljoin
import os.path

from django.conf import settings
from django.template import Library, TemplateSyntaxError

from ..thumbnail import ThumbError
from ..djangothumbnail import DjangoThumbnail


register = Library()


@register.filter
def sorethumb(file_field, processor_name):
    """ Returns the url path to a thumbnail for a given thumbnail processor. """

    try:
        processor = DjangoThumbnail.get_processor(processor_name)
    except ThumbError as e:
        raise TemplateSyntaxError(str(e))

    image_path = os.path.join(settings.MEDIA_ROOT, str(file_field))
    path = processor.process(image_path)

    return path



from setuptools import find_packages, setup

from sorethumb import __version__ as VERSION


setup(
    name='sorethumb',
    packages=find_packages(exclude=['docs']),
    version=VERSION,
    description='Thumbnail image processing, with Django integration',
    author='Will McGugan',
    author_email='will@willmcgugan.com',
    maintainer='Will McGugan',
    maintainer_email='will@willmcgugan.com',
    url='https://github.com/willmcgugan/sorethumb',
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=['pillow>=3.0.0'],
)
